@@ -289,9 +289,14 @@ static rrtype_descriptor_type rrtype_descriptors[(RRTYPE_DESCRIPTORS_LENGTH+1)]
 	/* 58 - TALINK */
 	{ 58, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
 	/* 59 - CDS */
-	{ 59, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
-	/* 60 */
-	{ 60, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	{ TYPE_CDS, "CDS", T_CDS, 4, 4,
+	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
+	  { RDATA_ZF_SHORT, RDATA_ZF_ALGORITHM, RDATA_ZF_BYTE, RDATA_ZF_HEX } },
+	/* 60 - CDNSKEY */
+	{ TYPE_CDNSKEY, "CDNSKEY", T_CDNSKEY, 4, 4,
+	  { RDATA_WF_SHORT, RDATA_WF_BYTE, RDATA_WF_BYTE, RDATA_WF_BINARY },
+	  { RDATA_ZF_SHORT, RDATA_ZF_BYTE, RDATA_ZF_ALGORITHM,
+	    RDATA_ZF_BASE64 } },
 	/* 61 */
 	{ 61, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
 	/* 62 */
@@ -725,7 +730,9 @@ static rrtype_descriptor_type rrtype_descriptors[(RRTYPE_DESCRIPTORS_LENGTH+1)]
 	/* 255 - * [RFC 1035, RFC 6895] */
 	{ 255, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
 	/* 256 - URI */
-	{ 256, NULL, T_UTYPE, 1, 1, { RDATA_WF_BINARY }, { RDATA_ZF_UNKNOWN } },
+	{ TYPE_URI, "URI", T_URI, 3, 3,
+	  { RDATA_WF_SHORT, RDATA_WF_SHORT, RDATA_WF_LONG_TEXT },
+	  { RDATA_ZF_SHORT, RDATA_ZF_SHORT, RDATA_ZF_LONG_TEXT } },
 	/* 257 - CAA [RFC 6844] */
 	{ TYPE_CAA, "CAA", T_CAA, 3, 3,
 	  { RDATA_WF_BYTE, RDATA_WF_TEXT, RDATA_WF_LONG_TEXT },
@@ -797,7 +804,7 @@ rrtype_from_string(const char *name)
 	rrtype_descriptor_type *entry;
 
 	/* Because this routine is called during zone parse for every record,
-	 * we optimise for frequently occuring records.
+	 * we optimise for frequently occurring records.
 	 * Also, we optimise for 'IN' and numbers are not rr types, because
 	 * during parse this routine is called for every rr class and TTL
 	 * to determine that it is not an RR type */
@@ -886,7 +893,7 @@ rrtype_from_string(const char *name)
 	if (strncasecmp(name, "TYPE", 4) != 0)
 		return 0;
 
-	if (!isdigit((int)name[4]))
+	if (!isdigit((unsigned char)name[4]))
 		return 0;
 
 	/* The rest from the string must be a number.  */
@@ -931,7 +938,7 @@ rrclass_from_string(const char *name)
 	if (strncasecmp(name, "CLASS", 5) != 0)
 		return 0;
 
-	if (!isdigit((int)name[5]))
+	if (!isdigit((unsigned char)name[5]))
 		return 0;
 
 	/* The rest from the string must be a number.  */
