@@ -249,7 +249,7 @@ do_deldomain(namedb_type* db, domain_type* domain)
 	 * one-smaller than this domain as closest-match. */
 	if(domain->parent->wildcard_child_closest_match == domain)
 		domain->parent->wildcard_child_closest_match =
-			domain_previous(domain);
+			domain_previous_existing_child(domain);
 
 	/* actual removal */
 	radix_delete(db->domains->nametree, domain->rnode);
@@ -443,20 +443,18 @@ domain_table_insert(domain_table_type* table,
 	return result;
 }
 
-int
-domain_table_iterate(domain_table_type* table,
-		    domain_table_iterator_type iterator,
-		    void* user_data)
+domain_type *domain_previous_existing_child(domain_type* domain)
 {
-	int error = 0;
-	struct radnode* n;
-	for(n = radix_first(table->nametree); n; n = radix_next(n)) {
-		error += iterator((domain_type*)n->elem, user_data);
+	domain_type* parent = domain->parent;
+	domain = domain_previous(domain);
+	while(domain && !domain->is_existing) {
+		if(domain == parent) /* do not walk back above parent */
+			return parent;
+		domain = domain_previous(domain);
 	}
-	return error;
+	return domain;
 }
 
-
 void
 domain_add_rrset(domain_type* domain, rrset_type* rrset)
 {
@@ -474,6 +472,15 @@ domain_add_rrset(domain_type* domain, rrset_type* rrset)
 
 	while (domain && !domain->is_existing) {
 		domain->is_existing = 1;
+		/* does this name in existance update the parent's
+		 * wildcard closest match? */
+		if(domain->parent
+		   && label_compare(dname_name(domain_dname(domain)),
+			(const uint8_t *) "\001*") <= 0
+		   && dname_compare(domain_dname(domain),
+		   	domain_dname(domain->parent->wildcard_child_closest_match)) > 0) {
+			domain->parent->wildcard_child_closest_match = domain;
+		}
 		domain = domain->parent;
 	}
 }
