@@ -344,6 +344,7 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o,
 		SERV_GET_BIN(debug_mode, o);
 		SERV_GET_BIN(do_ip4, o);
 		SERV_GET_BIN(do_ip6, o);
+		SERV_GET_BIN(reuseport, o);
 		SERV_GET_BIN(hide_version, o);
 		SERV_GET_BIN(zonefiles_check, o);
 		SERV_GET_BIN(log_time_ascii, o);
@@ -351,6 +352,7 @@ config_print_zone(nsd_options_t* opt, const char* k, int s, const char *o,
 		/* str */
 		SERV_GET_PATH(final, database, o);
 		SERV_GET_STR(identity, o);
+		SERV_GET_STR(version, o);
 		SERV_GET_STR(nsid, o);
 		SERV_GET_PATH(final, logfile, o);
 		SERV_GET_PATH(final, pidfile, o);
@@ -440,11 +442,13 @@ config_test_print_server(nsd_options_t* opt)
 	printf("server:\n");
 	printf("\tdebug-mode: %s\n", opt->debug_mode?"yes":"no");
 	printf("\tip-transparent: %s\n", opt->ip_transparent?"yes":"no");
+	printf("\treuseport: %s\n", opt->reuseport?"yes":"no");
 	printf("\tdo-ip4: %s\n", opt->do_ip4?"yes":"no");
 	printf("\tdo-ip6: %s\n", opt->do_ip6?"yes":"no");
 	printf("\thide-version: %s\n", opt->hide_version?"yes":"no");
 	print_string_var("database:", opt->database);
 	print_string_var("identity:", opt->identity);
+	print_string_var("version:", opt->version);
 	print_string_var("nsid:", opt->nsid);
 	print_string_var("logfile:", opt->logfile);
 	printf("\tserver_count: %d\n", opt->server_count);
@@ -539,18 +543,8 @@ file_inside_chroot(const char* fname, const char* chr)
 static int
 additional_checks(nsd_options_t* opt, const char* filename)
 {
-	ip_address_option_t* ip = opt->ip_addresses;
 	zone_options_t* zone;
-	int num = 0;
 	int errors = 0;
-	while(ip) {
-		num++;
-		ip = ip->next;
-	}
-	if(num > MAX_INTERFACES) {
-		fprintf(stderr, "%s: too many interfaces (ip-address:) specified.\n", filename);
-		errors ++;
-	}
 
 	RBTREE_FOR(zone, zone_options_t*, opt->zone_options)
 	{
@@ -565,6 +559,13 @@ additional_checks(nsd_options_t* opt, const char* filename)
 				"is received?\n", filename, zone->name);
 			errors ++;
 		}
+		if(!zone_is_slave(zone) && (!zone->pattern->zonefile ||
+			zone->pattern->zonefile[0] == 0)) {
+			fprintf(stderr, "%s: zone %s is a master zone but has "
+				"no zonefile. Where can the data come from?\n",
+				filename, zone->name);
+			errors ++;
+		}
 	}
 
 #ifndef BIND8_STATS
@@ -588,6 +589,11 @@ additional_checks(nsd_options_t* opt, const char* filename)
                       filename, (unsigned) strlen(opt->identity));
 		errors ++;
         }
+	if (opt->version && strlen(opt->version) > UCHAR_MAX) {
+                fprintf(stderr, "%s: server version too long (%u characters)\n",
+                      filename, (unsigned) strlen(opt->version));
+		errors ++;
+        }
 
 	/* not done here: parsing of ip-address. parsing of username. */
 
