@@ -108,6 +108,7 @@ struct nsd_socket
 {
 	struct addrinfo	*	addr;
 	int			s;
+	int			fam;
 };
 
 struct nsd_child
@@ -118,6 +119,9 @@ struct nsd_child
 	/* The child's process id.  */
 	pid_t pid;
 
+	/* child number in child array */
+	int child_num;
+
 	/*
 	 * Socket used by the parent process to send commands and
 	 * receive responses to/from this child process.
@@ -198,15 +202,17 @@ struct	nsd
 	unsigned char   *nsid;
 	uint8_t 		file_rotation_ok;
 
-	/* number of interfaces, ifs < MAX_INTERFACES */
+	/* number of interfaces */
 	size_t	ifs;
 	uint8_t grab_ip6_optional;
+	/* non0 if so_reuseport is in use, if so, tcp, udp array increased */
+	int reuseport;
 
-	/* TCP specific configuration */
-	struct nsd_socket tcp[MAX_INTERFACES];
+	/* TCP specific configuration (array size ifs) */
+	struct nsd_socket* tcp;
 
-	/* UDP specific configuration */
-	struct nsd_socket udp[MAX_INTERFACES];
+	/* UDP specific configuration (array size ifs) */
+	struct nsd_socket* udp;
 
 	edns_data_type edns_ipv4;
 #if defined(INET6)
@@ -248,6 +254,10 @@ struct	nsd
 	/* current zonestat array to use */
 	struct nsdst* zonestatnow;
 #endif /* BIND8_STATS */
+	/* ratelimit for errors, time value */
+	time_t err_limit_time;
+	/* ratelimit for errors, packet count */
+	unsigned int err_limit_count;
 
 	struct nsd_options* options;
 };
@@ -272,6 +282,8 @@ struct event_base* nsd_child_event_base(void);
 /* extra domain numbers for temporary domains */
 #define EXTRA_DOMAIN_NUMBERS 1024
 #define SLOW_ACCEPT_TIMEOUT 2 /* in seconds */
+/* ratelimit for error responses */
+#define ERROR_RATELIMIT 100 /* qps */
 /* allocate zonestat structures */
 void server_zonestat_alloc(struct nsd* nsd);
 /* remap the mmaps for zonestat isx, to bytesize sz.  Caller has to set
