@@ -132,7 +132,7 @@ struct query;
 #define	ARCOUNT(packet)		(buffer_read_u16_at((packet), 10))
 #define ARCOUNT_SET(packet, c)	(buffer_write_u16_at((packet), 10, (c)))
 
-/* Miscelaneous limits */
+/* Miscellaneous limits */
 #define MAX_PACKET_SIZE         65535   /* Maximum supported size of DNS packets.  */
 
 #define	QIOBUFSZ		(MAX_PACKET_SIZE + MAX_RR_SIZE)
@@ -196,4 +196,8 @@ int packet_read_query_section(buffer_type *packet,
 			uint16_t* qtype,
 			uint16_t* qclass);
 
+/* read notify SOA serial from packet. buffer position is unmodified on return.
+ * returns false on no-serial found or parse failure. */
+int packet_find_notify_serial(buffer_type *packet, uint32_t* serial);
+
 #endif /* _PACKET_H_ */
