@@ -135,6 +135,7 @@ query_error (struct query *q, nsd_rc_type rcode)
 	buffer_clear(q->packet);
 
 	QR_SET(q->packet);	   /* This is an answer.  */
+	AD_CLR(q->packet);
 	RCODE_SET(q->packet, (int) rcode); /* Error code.  */
 
 	/* Truncate the question as well... */
@@ -146,10 +147,28 @@ query_error (struct query *q, nsd_rc_type rcode)
 	return QUERY_PROCESSED;
 }
 
+static int
+query_ratelimit_err(nsd_type* nsd)
+{
+	time_t now = time(NULL);
+	if(nsd->err_limit_time == now) {
+		/* see if limit is exceeded for this second */
+		if(nsd->err_limit_count++ > ERROR_RATELIMIT)
+			return 1;
+	} else {
+		/* new second, new limits */
+		nsd->err_limit_time = now;
+		nsd->err_limit_count = 1;
+	}
+	return 0;
+}
+
 static query_state_type
-query_formerr (struct query *query)
+query_formerr (struct query *query, nsd_type* nsd)
 {
 	int opcode = OPCODE(query->packet);
+	if(query_ratelimit_err(nsd))
+		return QUERY_DISCARDED;
 	FLAGS_SET(query->packet, FLAGS(query->packet) & 0x0100U);
 			/* Preserve the RD flag. Clear the rest. */
 	OPCODE_SET(query->packet, opcode);
@@ -290,7 +309,6 @@ process_query_section(query_type *query)
 		&query->qtype, &query->qclass))
 		return 0;
 	query->qname = dname_make(query->region, qnamebuf, 1);
-	query->opcode = OPCODE(query->packet);
 	return 1;
 }
 
@@ -1089,7 +1107,7 @@ answer_authoritative(struct nsd   *nsd,
 		match = NULL;
 	}
 
-	/* Authorative zone.  */
+	/* Authoritative zone.  */
 #ifdef NSEC3
 	if (q->edns.dnssec_ok && q->zone->nsec3_param) {
 		nsec3_answer_authoritative(&match, q, answer,
@@ -1185,7 +1203,7 @@ answer_lookup_zone(struct nsd *nsd, struct query *q, answer_type *answer,
 	if (exact && q->qtype == TYPE_DS && closest_encloser == q->zone->apex) {
 		/*
 		 * Type DS query at the zone apex (and the server is
-		 * not authoratitive for the parent zone).
+		 * not authoritative for the parent zone).
 		 */
 		if (q->qclass == CLASS_ANY) {
 			AA_CLR(q->packet);
@@ -1249,7 +1267,7 @@ answer_query(struct nsd *nsd, struct query *q)
 }
 
 void
-query_prepare_response(query_type *q)
+query_prepare_response(query_type *q, nsd_type *nsd)
 {
 	uint16_t flags;
 
@@ -1264,6 +1282,9 @@ query_prepare_response(query_type *q)
 	 */
 	q->reserved_space = edns_reserved_space(&q->edns);
 	q->reserved_space += tsig_reserved_space(&q->tsig);
+	if(q->edns.nsid == 1 && nsd->nsid_len > 0 &&
+		q->edns.status != EDNS_NOT_PRESENT)
+		q->reserved_space += OPT_HDR + nsd->nsid_len;
 
 	/* Update the flags.  */
 	flags = FLAGS(q->packet);
@@ -1296,8 +1317,17 @@ query_process(query_type *q, nsd_type *nsd)
 		return QUERY_DISCARDED;
 	}
 
+	/* check opcode early on, because new opcodes may have different
+	 * specification of the meaning of the rest of the packet */
+	q->opcode = OPCODE(q->packet);
+	if(q->opcode != OPCODE_QUERY && q->opcode != OPCODE_NOTIFY) {
+		if(query_ratelimit_err(nsd))
+			return QUERY_DISCARDED;
+		return query_error(q, NSD_RC_IMPL);
+	}
+
 	if (RCODE(q->packet) != RCODE_OK || !process_query_section(q)) {
-		return query_formerr(q);
+		return query_formerr(q, nsd);
 	}
 
 	/* Update statistics.  */
@@ -1309,6 +1339,8 @@ query_process(query_type *q, nsd_type *nsd)
 		if (q->opcode == OPCODE_NOTIFY) {
 			return answer_notify(nsd, q);
 		} else {
+			if(query_ratelimit_err(nsd))
+				return QUERY_DISCARDED;
 			return query_error(q, NSD_RC_IMPL);
 		}
 	}
@@ -1316,7 +1348,7 @@ query_process(query_type *q, nsd_type *nsd)
 	/* Dont bother to answer more than one question at once... */
 	if (QDCOUNT(q->packet) != 1) {
 		FLAGS_SET(q->packet, 0);
-		return query_formerr(q);
+		return query_formerr(q, nsd);
 	}
 	/* Ignore settings of flags */
 
@@ -1324,13 +1356,13 @@ query_process(query_type *q, nsd_type *nsd)
 	   except for IXFR queries. */
 	if (ANCOUNT(q->packet) != 0 ||
 		(q->qtype!=TYPE_IXFR && NSCOUNT(q->packet) != 0)) {
-		return query_formerr(q);
+		return query_formerr(q, nsd);
 	}
 	if(q->qtype==TYPE_IXFR && NSCOUNT(q->packet) > 0) {
 		int i; /* skip ixfr soa information data here */
 		for(i=0; i< NSCOUNT(q->packet); i++)
 			if(!packet_skip_rr(q->packet, 0))
-				return query_formerr(q);
+				return query_formerr(q, nsd);
 	}
 
 	arcount = ARCOUNT(q->packet);
@@ -1346,7 +1378,7 @@ query_process(query_type *q, nsd_type *nsd)
 
 		/* see if tsig is before edns record */
 		if (!tsig_parse_rr(&q->tsig, q->packet))
-			return query_formerr(q);
+			return query_formerr(q, nsd);
 		if(q->tsig.status != TSIG_NOT_PRESENT)
 			--arcount;
 	}
@@ -1359,20 +1391,20 @@ query_process(query_type *q, nsd_type *nsd)
 	if (arcount > 0 && q->tsig.status == TSIG_NOT_PRESENT) {
 		/* see if tsig is after the edns record */
 		if (!tsig_parse_rr(&q->tsig, q->packet))
-			return query_formerr(q);
+			return query_formerr(q, nsd);
 		if(q->tsig.status != TSIG_NOT_PRESENT)
 			--arcount;
 	}
 	/* If more RRs left in Add. Section, FORMERR. */
 	if (arcount > 0) {
-		return query_formerr(q);
+		return query_formerr(q, nsd);
 	}
 
 	/* Do we have any trailing garbage? */
 #ifdef	STRICT_MESSAGE_PARSE
 	if (buffer_remaining(q->packet) > 0) {
 		/* If we're strict.... */
-		return query_formerr(q);
+		return query_formerr(q, nsd);
 	}
 #endif
 	/* Remove trailing garbage.  */
@@ -1391,7 +1423,7 @@ query_process(query_type *q, nsd_type *nsd)
 		return query_error(q, NSD_RC_OK);
 	}
 
-	query_prepare_response(q);
+	query_prepare_response(q, nsd);
 
 	if (q->qclass != CLASS_IN && q->qclass != CLASS_ANY) {
 		if (q->qclass == CLASS_CH) {
@@ -1430,8 +1462,8 @@ query_add_optional(query_type *q, nsd_type *nsd)
 		if (q->edns.dnssec_ok)	edns->ok[7] = 0x80;
 		else			edns->ok[7] = 0x00;
 		buffer_write(q->packet, edns->ok, OPT_LEN);
-		if (nsd->nsid_len > 0 && q->edns.nsid == 1 &&
-				!query_overflow_nsid(q, nsd->nsid_len)) {
+		if (nsd->nsid_len > 0 && q->edns.nsid == 1 && buffer_available(
+			q->packet, OPT_RDATA+OPT_HDR+nsd->nsid_len)) {
 			/* rdata length */
 			buffer_write(q->packet, edns->rdata_nsid, OPT_RDATA);
 			/* nsid opt header */
