@@ -263,12 +263,14 @@ daemon_remote_create(nsd_options_t* cfg)
 		return NULL;
 	}
 	/* no SSLv2, SSLv3 because has defects */
-	if(!(SSL_CTX_set_options(rc->ctx, SSL_OP_NO_SSLv2) & SSL_OP_NO_SSLv2)){
+	if((SSL_CTX_set_options(rc->ctx, SSL_OP_NO_SSLv2) & SSL_OP_NO_SSLv2)
+		!= SSL_OP_NO_SSLv2){
 		log_crypto_err("could not set SSL_OP_NO_SSLv2");
 		daemon_remote_delete(rc);
 		return NULL;
 	}
-	if(!(SSL_CTX_set_options(rc->ctx, SSL_OP_NO_SSLv3) & SSL_OP_NO_SSLv3)){
+	if((SSL_CTX_set_options(rc->ctx, SSL_OP_NO_SSLv3) & SSL_OP_NO_SSLv3)
+		!= SSL_OP_NO_SSLv3){
 		log_crypto_err("could not set SSL_OP_NO_SSLv3");
 		daemon_remote_delete(rc);
 		return NULL;
@@ -832,7 +834,7 @@ do_notify(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 		struct notify_zone_t* n = (struct notify_zone_t*)rbtree_search(
 			xfrd->notify_zones, (const dname_type*)zo->node.key);
 		if(n) {
-			xfrd_notify_start(n);
+			xfrd_notify_start(n, xfrd);
 			send_ok(ssl);
 		} else {
 			ssl_printf(ssl, "error zone does not have notify\n");
@@ -840,7 +842,7 @@ do_notify(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	} else {
 		struct notify_zone_t* n;
 		RBTREE_FOR(n, struct notify_zone_t*, xfrd->notify_zones) {
-			xfrd_notify_start(n);
+			xfrd_notify_start(n, xfrd);
 		}
 		send_ok(ssl);
 	}
@@ -1105,15 +1107,15 @@ zonestat_inc_ifneeded(xfrd_state_t* xfrd)
 #endif /* USE_ZONE_STATS */
 }
 
-/** do the addzone command */
-static void
-do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
+/** perform the addzone command for one zone */
+static int
+perform_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 {
 	const dname_type* dname;
 	zone_options_t* zopt;
 	char* arg2 = NULL;
 	if(!find_arg2(ssl, arg, &arg2))
-		return;
+		return 0;
 
 	/* if we add it to the xfrd now, then xfrd could download AXFR and
 	 * store it and the NSD-reload would see it in the difffile before
@@ -1127,13 +1129,13 @@ do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	if(!rbtree_search(xfrd->nsd->options->patterns, arg2)) {
 		(void)ssl_printf(ssl, "error pattern %s does not exist\n",
 			arg2);
-		return;
+		return 0;
 	}
 
 	dname = dname_parse(xfrd->region, arg);
 	if(!dname) {
 		(void)ssl_printf(ssl, "error cannot parse zone name\n");
-		return;
+		return 0;
 	}
 
 	/* see if zone is a duplicate */
@@ -1141,8 +1143,7 @@ do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 		region_recycle(xfrd->region, (void*)dname,
 			dname_total_size(dname));
 		(void)ssl_printf(ssl, "zone %s already exists\n", arg);
-		send_ok(ssl); /* a nop operation */
-		return;
+		return 1;
 	}
 	region_recycle(xfrd->region, (void*)dname, dname_total_size(dname));
 	dname = NULL;
@@ -1152,7 +1153,7 @@ do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	if(!zopt) {
 		/* also dname parse error here */
 		(void)ssl_printf(ssl, "error could not add zonelist entry\n");
-		return;
+		return 0;
 	}
 	/* make addzone task and schedule reload */
 	task_new_add_zone(xfrd->nsd->task[xfrd->nsd->mytask],
@@ -1166,13 +1167,12 @@ do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	if(zone_is_slave(zopt)) {
 		xfrd_init_slave_zone(xfrd, zopt);
 	}
-
-	send_ok(ssl);
+	return 1;
 }
 
-/** do the delzone command */
-static void
-do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
+/** perform the delzone command for one zone */
+static int
+perform_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 {
 	const dname_type* dname;
 	zone_options_t* zopt;
@@ -1180,7 +1180,7 @@ do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	dname = dname_parse(xfrd->region, arg);
 	if(!dname) {
 		(void)ssl_printf(ssl, "error cannot parse zone name\n");
-		return;
+		return 0;
 	}
 
 	/* see if we have the zone in question */
@@ -1190,9 +1190,8 @@ do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 			dname_total_size(dname));
 		/* nothing to do */
 		if(!ssl_printf(ssl, "warning zone %s not present\n", arg))
-			return;
-		send_ok(ssl);
-		return;
+			return 0;
+		return 1;
 	}
 
 	/* see if it can be deleted */
@@ -1202,7 +1201,7 @@ do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 		(void)ssl_printf(ssl, "error zone defined in nsd.conf, "
 			"cannot delete it in this manner: remove it from "
 			"nsd.conf yourself and repattern\n");
-		return;
+		return 0;
 	}
 
 	/* create deletion task */
@@ -1218,9 +1217,72 @@ do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
 	zone_list_del(xfrd->nsd->options, zopt);
 
 	region_recycle(xfrd->region, (void*)dname, dname_total_size(dname));
+	return 1;
+}
+
+/** do the addzone command */
+static void
+do_addzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
+{
+	if(!perform_addzone(ssl, xfrd, arg))
+		return;
 	send_ok(ssl);
 }
 
+/** do the delzone command */
+static void
+do_delzone(SSL* ssl, xfrd_state_t* xfrd, char* arg)
+{
+	if(!perform_delzone(ssl, xfrd, arg))
+		return;
+	send_ok(ssl);
+}
+
+/** do the addzones command */
+static void
+do_addzones(SSL* ssl, xfrd_state_t* xfrd)
+{
+	char buf[2048];
+	int num = 0;
+	while(ssl_read_line(ssl, buf, sizeof(buf))) {
+		if(buf[0] == 0x04 && buf[1] == 0)
+			break; /* end of transmission */
+		if(!perform_addzone(ssl, xfrd, buf)) {
+			if(!ssl_printf(ssl, "error for input line '%s'\n", 
+				buf))
+				return;
+		} else {
+			if(!ssl_printf(ssl, "added: %s\n", buf))
+				return;
+			num++;
+		}
+	}
+	(void)ssl_printf(ssl, "added %d zones\n", num);
+}
+
+/** do the delzones command */
+static void
+do_delzones(SSL* ssl, xfrd_state_t* xfrd)
+{
+	char buf[2048];
+	int num = 0;
+	while(ssl_read_line(ssl, buf, sizeof(buf))) {
+		if(buf[0] == 0x04 && buf[1] == 0)
+			break; /* end of transmission */
+		if(!perform_delzone(ssl, xfrd, buf)) {
+			if(!ssl_printf(ssl, "error for input line '%s'\n", 
+				buf))
+				return;
+		} else {
+			if(!ssl_printf(ssl, "removed: %s\n", buf))
+				return;
+			num++;
+		}
+	}
+	(void)ssl_printf(ssl, "deleted %d zones\n", num);
+}
+
+
 /** remove TSIG key from config and add task so that reload does too */
 static void remove_key(xfrd_state_t* xfrd, const char* kname)
 {
@@ -1433,7 +1495,7 @@ repat_interrupt_notify_start(xfrd_state_t* xfrd)
 				nz->notify_current = nz->options->pattern->notify;
 			if(nz->notify_restart == 2) {
 				if(nz->notify_restart)
-					xfrd_notify_start(nz);
+					xfrd_notify_start(nz, xfrd);
 			}
 		}
 	}
@@ -1661,6 +1723,10 @@ execute_cmd(struct daemon_remote* rc, SSL* ssl, char* cmd, struct rc_state* rs)
 		do_addzone(ssl, rc->xfrd, skipwhite(p+7));
 	} else if(cmdcmp(p, "delzone", 7)) {
 		do_delzone(ssl, rc->xfrd, skipwhite(p+7));
+	} else if(cmdcmp(p, "addzones", 8)) {
+		do_addzones(ssl, rc->xfrd);
+	} else if(cmdcmp(p, "delzones", 8)) {
+		do_delzones(ssl, rc->xfrd);
 	} else if(cmdcmp(p, "notify", 6)) {
 		do_notify(ssl, rc->xfrd, skipwhite(p+6));
 	} else if(cmdcmp(p, "transfer", 8)) {
