@@ -23,7 +23,7 @@
 #include "xfrd-disk.h"
 #include "util.h"
 
-/* sort tcppipe, first on IP address, for an IPadresss, sort on num_unused */
+/* sort tcppipe, first on IP address, for an IPaddresss, sort on num_unused */
 static int
 xfrd_pipe_cmp(const void* a, const void* b)
 {
@@ -568,7 +568,10 @@ xfrd_tcp_setup_write_packet(struct xfrd_tcp_pipeline* tp, xfrd_zone_t* zone)
 	assert(zone->tcp_waiting == 0);
 	/* start AXFR or IXFR for the zone */
 	if(zone->soa_disk_acquired == 0 || zone->master->use_axfr_only ||
-						zone->master->ixfr_disabled) {
+		zone->master->ixfr_disabled ||
+		/* if zone expired, after the first round, do not ask for
+		 * IXFR any more, but full AXFR (of any serial number) */
+		(zone->state == xfrd_zone_expired && zone->round_num != 0)) {
 		DEBUG(DEBUG_XFRD,1, (LOG_INFO, "request full zone transfer "
 						"(AXFR) for %s to %s",
 			zone->apex_str, zone->master->ip_address_spec));
@@ -905,7 +908,7 @@ xfrd_tcp_release(xfrd_tcp_set_t* set, xfrd_zone_t* zone)
 		socklen_t to_len = xfrd_acl_sockaddr_to(
 			set->tcp_waiting_first->master, &to);
 		if(to_len == tp->ip_len && memcmp(&to, &tp->ip, to_len) == 0) {
-			/* use this connnection for the waiting zone */
+			/* use this connection for the waiting zone */
 			zone = set->tcp_waiting_first;
 			assert(zone->tcp_conn == -1);
 			zone->tcp_conn = conn;
