@@ -65,6 +65,13 @@ struct xfrd_state {
 	/* counter for xfr file numbers */
 	uint64_t xfrfilenumber;
 
+	/* the zonestat array size that we last saw and is safe to use */
+	unsigned zonestat_safe;
+	/* size currently of the clear array */
+	size_t zonestat_clear_num;
+	/* array of malloced entries with cumulative cleared stat values */
+	struct nsdst** zonestat_clear;
+
 	/* timer for NSD reload */
 	struct timeval reload_timeout;
 	struct event reload_handler;
@@ -342,6 +349,9 @@ void xfrd_process_task_result(xfrd_state_t* xfrd, struct udb_base* taskudb);
 /* set to reload right away (for user controlled reload events) */
 void xfrd_set_reload_now(xfrd_state_t* xfrd);
 
+/* send expiry notifications to nsd */
+void xfrd_send_expire_notification(xfrd_zone_t* zone);
+
 /* handle incoming notify (soa or NULL) and start zone xfr if necessary */
 void xfrd_handle_notify_and_start_xfr(xfrd_zone_t* zone, xfrd_soa_t* soa);
 
