@@ -333,7 +333,7 @@ zparser_conv_algorithm(region_type *region, const char *text)
 uint16_t *
 zparser_conv_certificate_type(region_type *region, const char *text)
 {
-	/* convert a algoritm string to integer */
+	/* convert an algorithm string to integer */
 	const lookup_table_type *type;
 	uint16_t id;
 
@@ -1434,7 +1434,10 @@ process_rr(void)
 		rr_type* o;
 		if (rr->type != TYPE_RRSIG && rrset->rrs[0].ttl != rr->ttl) {
 			zc_warning_prev_line(
-				"TTL does not match the TTL of the RRset");
+				"%s TTL %u does not match the TTL %u of the %s RRset",
+				domain_to_string(rr->owner), (unsigned)rr->ttl,
+				(unsigned)rrset->rrs[0].ttl,
+				rrtype_to_string(rr->type));
 		}
 
 		/* Search for possible duplicates... */
@@ -1449,7 +1452,7 @@ process_rr(void)
 			return 0;
 		}
 		if(rrset->rr_count == 65535) {
-			zc_error_prev_line("too may RRs for domain RRset");
+			zc_error_prev_line("too many RRs for domain RRset");
 			return 0;
 		}
 
